@@ -18,16 +18,20 @@
 # You should have received a copy of the GNU General Public License
 # along with PyEPR.  If not, see <http://www.gnu.org/licenses/>.
 
+
 import os
 import re
 import sys
+import numbers
 import unittest
 import tempfile
 import functools
 
 import numpy as np
 
-sys.path.insert(0, os.pardir)
+TESTDIR = os.path.abspath(os.path.dirname(__file__))
+sys.path.insert(0, os.path.join(TESTDIR, os.pardir))
+
 import epr
 
 
@@ -49,6 +53,7 @@ EPR_TO_NUMPY_TYPE = {
 
 TEST_PRODUCT = 'MER_LRC_2PTGMV20000620_104318_00000104X000_00000_00000_0001.N1'
 
+
 def quiet(func):
     @functools.wraps(func)
     def wrapper(*args, **kwds):
@@ -58,7 +63,7 @@ def quiet(func):
             # using '/dev/null' doesn't work in python 3 because the file
             # object coannot be converted into a C FILE*
             #with file(os.devnull) as fd:
-            with tempfile.TemporaryFile() as fd:
+            with tempfile.TemporaryFile('w+') as fd:
                 sys.stdout = fd
                 sys.stderr = fd
                 ret = func(*args, **kwds)
@@ -69,6 +74,7 @@ def quiet(func):
 
     return wrapper
 
+
 def equal_products(product1, product2):
     if type(product1) != type(product2):
         return False
@@ -86,12 +92,27 @@ def equal_products(product1, product2):
 
 
 class TestOpenProduct(unittest.TestCase):
-    PRODUCT_FILE = TEST_PRODUCT
+    PRODUCT_FILE = os.path.join(TESTDIR, TEST_PRODUCT)
 
     def test_open(self):
         product = epr.open(self.PRODUCT_FILE)
         self.assertTrue(isinstance(product, epr.Product))
 
+    if 'unicode' in dir(__builtins__):
+
+        def test_open_unicode(self):
+            filename = unicode(self.PRODUCT_FILE)
+            product = epr.open(filename)
+            self.assertTrue(isinstance(product, epr.Product))
+
+    else:
+
+        def test_open_bytes(self):
+            filename = self.PRODUCT_FILE.encode('UTF-8')
+            product = epr.open(filename)
+            self.assertTrue(isinstance(filename, bytes))
+            self.assertTrue(isinstance(product, epr.Product))
+
     def test_product_constructor(self):
         product = epr.Product(self.PRODUCT_FILE)
         self.assertTrue(isinstance(product, epr.Product))
@@ -104,7 +125,7 @@ class TestOpenProduct(unittest.TestCase):
 
 
 class TestProduct(unittest.TestCase):
-    PRODUCT_FILE = TEST_PRODUCT
+    PRODUCT_FILE = os.path.join(TESTDIR, TEST_PRODUCT)
     ID_STRING = 'MER_LRC_2PTGMV20000620_104318_00000104X000_00000'
     TOT_SIZE = 407461
 
@@ -127,12 +148,11 @@ class TestProduct(unittest.TestCase):
     SPH_DESCRIPTOR = 'MER_LRC_2P SPECIFIC HEADER'
     MERIS_IODD_VERSION = 7
 
-
     def setUp(self):
         self.product = epr.Product(self.PRODUCT_FILE)
 
     def test_file_path_property(self):
-        self.assertEqual(self.product.file_path, TEST_PRODUCT)
+        self.assertEqual(self.product.file_path, self.PRODUCT_FILE)
 
     def test_tot_size_property(self):
         self.assertEqual(self.product.tot_size, self.TOT_SIZE)
@@ -141,7 +161,8 @@ class TestProduct(unittest.TestCase):
         self.assertEqual(self.product.id_string, self.ID_STRING)
 
     def test_meris_iodd_version_property(self):
-        self.assertEqual(self.product.meris_iodd_version, self.MERIS_IODD_VERSION)
+        self.assertEqual(self.product.meris_iodd_version,
+                         self.MERIS_IODD_VERSION)
 
     def test_get_scene_width(self):
         self.assertEqual(self.product.get_scene_width(), self.DATASET_WIDTH)
@@ -167,6 +188,20 @@ class TestProduct(unittest.TestCase):
         dataset = self.product.get_dataset(self.DATASET_NAME)
         self.assertTrue(dataset)
 
+    if 'unicode' in dir(__builtins__):
+
+        def test_get_dataset_unicode(self):
+            dataset = self.product.get_dataset(unicode(self.DATASET_NAME))
+            self.assertTrue(dataset)
+
+    else:
+
+        def test_get_dataset_byrtes(self):
+            filename = self.DATASET_NAME.encode('UTF-8')
+            dataset = self.product.get_dataset(filename)
+            self.assertTrue(isinstance(filename, bytes))
+            self.assertTrue(dataset)
+
     def test_datasets(self):
         datasets = [self.product.get_dataset_at(idx)
                         for idx in range(self.product.get_num_datasets())]
@@ -179,19 +214,35 @@ class TestProduct(unittest.TestCase):
     def test_get_mph(self):
         record = self.product.get_mph()
         self.assertTrue(isinstance(record, epr.Record))
-        self.assertEqual(record.get_field('PRODUCT').get_elem(),
-                         self.PRODUCT_FILE)
+        product = record.get_field('PRODUCT').get_elem()
+        self.assertEqual(product.decode('ascii'),
+                        os.path.basename(self.PRODUCT_FILE))
 
     def test_get_sph(self):
         record = self.product.get_sph()
         self.assertTrue(isinstance(record, epr.Record))
-        self.assertEqual(record.get_field('SPH_DESCRIPTOR').get_elem(),
-                         self.SPH_DESCRIPTOR)
+        sph_desct = record.get_field('SPH_DESCRIPTOR').get_elem()
+        self.assertEqual(sph_desct.decode('ascii'), self.SPH_DESCRIPTOR)
 
     def test_get_band_id(self):
         self.assertTrue(isinstance(self.product.get_band(self.BAND_NAME),
                                    epr.Band))
 
+    if 'unicode' in dir(__builtins__):
+
+        def test_get_band_id_unicode(self):
+            band_name = unicode(self.BAND_NAME)
+            self.assertTrue(isinstance(self.product.get_band(band_name),
+                                       epr.Band))
+
+    else:
+
+        def test_get_band_id_bytes(self):
+            band_name = self.BAND_NAME.encode('UTF-8')
+            self.assertTrue(isinstance(band_name, bytes))
+            self.assertTrue(isinstance(self.product.get_band(self.BAND_NAME),
+                                       epr.Band))
+
     def test_get_band_id_invalid_name(self):
         self.assertRaises(ValueError, self.product.get_band, '')
 
@@ -205,10 +256,10 @@ class TestProduct(unittest.TestCase):
     def test_read_bitmask_raster(self):
         bm_expr = 'l2_flags.LAND AND !l2_flags.BRIGHT'
 
-        xoffset = self.DATASET_WIDTH / 2
-        yoffset = self.DATASET_HEIGHT / 2
-        width = self.DATASET_WIDTH / 2
-        height = self.DATASET_HEIGHT / 2
+        xoffset = self.DATASET_WIDTH // 2
+        yoffset = self.DATASET_HEIGHT // 2
+        width = self.DATASET_WIDTH // 2
+        height = self.DATASET_HEIGHT // 2
 
         raster = epr.create_bitmask_raster(width, height)
         raster = self.product.read_bitmask_raster(bm_expr, xoffset, yoffset,
@@ -217,13 +268,48 @@ class TestProduct(unittest.TestCase):
         self.assertEqual(raster.get_width(), width)
         self.assertEqual(raster.get_height(), height)
 
+    if 'unicode' in dir(__builtins__):
+
+        def test_read_bitmask_raster_unicode(self):
+            bm_expr = unicode('l2_flags.LAND AND !l2_flags.BRIGHT')
+
+            xoffset = self.DATASET_WIDTH // 2
+            yoffset = self.DATASET_HEIGHT // 2
+            width = self.DATASET_WIDTH // 2
+            height = self.DATASET_HEIGHT // 2
+
+            raster = epr.create_bitmask_raster(width, height)
+            raster = self.product.read_bitmask_raster(bm_expr,
+                                                      xoffset, yoffset, raster)
+            self.assertTrue(isinstance(raster, epr.Raster))
+            self.assertEqual(raster.get_width(), width)
+            self.assertEqual(raster.get_height(), height)
+
+    else:
+
+        def test_read_bitmask_raster_bytes(self):
+            bm_expr = 'l2_flags.LAND AND !l2_flags.BRIGHT'.encode('UTF-8')
+            self.assertTrue(isinstance(bm_expr, bytes))
+
+            xoffset = self.DATASET_WIDTH // 2
+            yoffset = self.DATASET_HEIGHT // 2
+            width = self.DATASET_WIDTH // 2
+            height = self.DATASET_HEIGHT // 2
+
+            raster = epr.create_bitmask_raster(width, height)
+            raster = self.product.read_bitmask_raster(bm_expr,
+                                                      xoffset, yoffset, raster)
+            self.assertTrue(isinstance(raster, epr.Raster))
+            self.assertEqual(raster.get_width(), width)
+            self.assertEqual(raster.get_height(), height)
+
     def test_read_bitmask_raster_with_invalid_bm_expr(self):
         bm_expr = 'l5_flags.LAND AND !l2_flags.BRIGHT'
 
-        xoffset = self.DATASET_WIDTH / 2
-        yoffset = self.DATASET_HEIGHT / 2
-        width = self.DATASET_WIDTH / 2
-        height = self.DATASET_HEIGHT / 2
+        xoffset = self.DATASET_WIDTH // 2
+        yoffset = self.DATASET_HEIGHT // 2
+        width = self.DATASET_WIDTH // 2
+        height = self.DATASET_HEIGHT // 2
 
         raster = epr.create_bitmask_raster(width, height)
         self.assertRaises(epr.EPRError, self.product.read_bitmask_raster,
@@ -236,10 +322,10 @@ class TestProduct(unittest.TestCase):
     def test_read_bitmask_raster_with_wrong_data_type(self):
         bm_expr = 'l2_flags.LAND AND !l2_flags.BRIGHT'
 
-        xoffset = self.DATASET_WIDTH / 2
-        yoffset = self.DATASET_HEIGHT / 2
-        width = self.DATASET_WIDTH / 2
-        height = self.DATASET_HEIGHT / 2
+        xoffset = self.DATASET_WIDTH // 2
+        yoffset = self.DATASET_HEIGHT // 2
+        width = self.DATASET_WIDTH // 2
+        height = self.DATASET_HEIGHT // 2
 
         raster = epr.create_raster(epr.E_TID_DOUBLE, width, height)
         self.assertRaises(epr.EPRError, self.product.read_bitmask_raster,
@@ -251,7 +337,7 @@ class TestProduct(unittest.TestCase):
 
 
 class TestProductHighLevelAPI(unittest.TestCase):
-    PRODUCT_FILE = TEST_PRODUCT
+    PRODUCT_FILE = os.path.join(TESTDIR, TEST_PRODUCT)
     DATASET_NAMES = TestProduct.DATASET_NAMES
     BAND_NAMES = [
         'latitude',
@@ -317,6 +403,9 @@ class TestProductHighLevelAPI(unittest.TestCase):
         self.assertEqual(mobj.group('n_bands'),
                          str(self.product.get_num_bands()))
 
+    def test_repr_type(self):
+        self.assertTrue(isinstance(repr(self.product), str))
+
     def test_str(self):
         lines = [repr(self.product), '']
         lines.extend(map(repr, self.product.datasets()))
@@ -325,9 +414,12 @@ class TestProductHighLevelAPI(unittest.TestCase):
         data = '\n'.join(lines)
         self.assertEqual(data, str(self.product))
 
+    def test_str_type(self):
+        self.assertTrue(isinstance(str(self.product), str))
+
 
 class TestDataset(unittest.TestCase):
-    PRODUCT_FILE = TEST_PRODUCT
+    PRODUCT_FILE = os.path.join(TESTDIR, TEST_PRODUCT)
     DATASET_NAME = 'Vapour_Content'
     DATASET_DESCRIPTION = 'Level 2 MDS Total Water vapour'
     NUM_RECORDS = 149
@@ -372,7 +464,7 @@ class TestDataset(unittest.TestCase):
 
 
 class TestDatasetHighLevelAPI(unittest.TestCase):
-    PRODUCT_FILE = TEST_PRODUCT
+    PRODUCT_FILE = os.path.join(TESTDIR, TEST_PRODUCT)
     DATASET_NAME = TestDataset.DATASET_NAME
 
     def setUp(self):
@@ -402,7 +494,11 @@ class TestDatasetHighLevelAPI(unittest.TestCase):
         mobj = re.match(pattern, repr(self.dataset))
         self.assertNotEqual(mobj, None)
         self.assertEqual(mobj.group('name'), self.dataset.get_name())
-        self.assertEqual(mobj.group('num'), str(self.dataset.get_num_records()))
+        self.assertEqual(mobj.group('num'),
+                         str(self.dataset.get_num_records()))
+
+    def test_repr_type(self):
+        self.assertTrue(isinstance(repr(self.dataset), str))
 
     def test_str(self):
         lines = [repr(self.dataset), '']
@@ -410,9 +506,12 @@ class TestDatasetHighLevelAPI(unittest.TestCase):
         data = '\n'.join(lines)
         self.assertEqual(data, str(self.dataset))
 
+    def test_str_type(self):
+        self.assertTrue(isinstance(str(self.dataset), str))
+
 
 class TestBand(unittest.TestCase):
-    PRODUCT_FILE = TEST_PRODUCT
+    PRODUCT_FILE = os.path.join(TESTDIR, TEST_PRODUCT)
     BAND_NAMES = (
         'latitude',
         'longitude',
@@ -546,8 +645,8 @@ class TestBand(unittest.TestCase):
         src_height = self.product.get_scene_height()
         xstep = 2
         ystep = 3
-        width = (src_width - 1) / xstep + 1
-        height = (src_height - 1) / ystep + 1
+        width = (src_width - 1) // xstep + 1
+        height = (src_height - 1) // ystep + 1
         raster = self.band.create_compatible_raster(src_width, src_height,
                                                     xstep, ystep)
         self.assertTrue(isinstance(raster, epr.Raster))
@@ -602,8 +701,8 @@ class TestBand(unittest.TestCase):
         self.band.read_raster(raster=raster2)
 
         self.assertEqual(raster1.get_pixel(0, 0), raster2.get_pixel(0, 0))
-        self.assertEqual(raster1.get_pixel(width - 1, height -1),
-                          raster2.get_pixel(width - 1, height -1))
+        self.assertEqual(raster1.get_pixel(width - 1, height - 1),
+                          raster2.get_pixel(width - 1, height - 1))
 
     def test_read_raster_with_invalid_raster(self):
         self.assertRaises(TypeError, self.band.read_raster, 0, 0, 0)
@@ -634,7 +733,9 @@ class TestBand(unittest.TestCase):
         h, w = self.TEST_DATA.shape
         self.assertTrue(np.allclose(data[:h, :w], self.TEST_DATA))
 
-    # @TODO: check, it seems to be an upstream bug or a metter of data mirroring
+    # @TODO: check, it seems to be an upstream bug or a metter of data
+    #        mirroring
+    # @SEEALSO: https://www.brockmann-consult.de/beam-jira/browse/EPR-2
     #def test_read_as_array_with_step(self):
     #    width = 400
     #    height = 300
@@ -647,14 +748,16 @@ class TestBand(unittest.TestCase):
     #    self.assertEqual(data.dtype, self.DATA_TYPE)
     #
     #    h, w = self.TEST_DATA.shape
-    #    self.assertTrue(np.all(data[:h/2, :w/2] == self.TEST_DATA[::2, ::2]))
-    #    #self.assertTrue(np.all(data[:h/2, :w/2] == self.TEST_DATA[::2, 1::2]))
+    #    self.assertTrue(
+    #               np.all(data[:h / 2, :w / 2] == self.TEST_DATA[::2, ::2]))
+    #    #self.assertTrue(
+    #    #           np.all(data[:h / 2, :w / 2] == self.TEST_DATA[::2, 1::2]))
 
     # @TODO: more read_as_array testing
 
 
 class TestBandHighLevelAPI(unittest.TestCase):
-    PRODUCT_FILE = TEST_PRODUCT
+    PRODUCT_FILE = os.path.join(TESTDIR, TEST_PRODUCT)
 
     def setUp(self):
         self.product = epr.Product(self.PRODUCT_FILE)
@@ -668,6 +771,14 @@ class TestBandHighLevelAPI(unittest.TestCase):
             self.assertEqual(mobj.group('name'), band.get_name())
             self.assertEqual(mobj.group('product_id'), self.product.id_string)
 
+    def test_repr_type(self):
+        band = self.product.get_band_at(0)
+        self.assertTrue(isinstance(repr(band), str))
+
+    def test_str_type(self):
+        band = self.product.get_band_at(0)
+        self.assertTrue(isinstance(str(band), str))
+
 
 class TestCreateRaster(unittest.TestCase):
     RASTER_WIDTH = 400
@@ -700,7 +811,7 @@ class TestCreateRaster(unittest.TestCase):
 
     def test_create_raster_with_invalid_size(self):
         self.assertRaises((ValueError, OverflowError), epr.create_raster,
-                          self.RASTER_DATA_TYPE,  -1, self.RASTER_HEIGHT)
+                          self.RASTER_DATA_TYPE, -1, self.RASTER_HEIGHT)
 
     def test_create_bitmask_raster(self):
         raster = epr.create_bitmask_raster(self.RASTER_WIDTH,
@@ -751,7 +862,8 @@ class TestRaster(unittest.TestCase):
         self.assertEqual(self.raster.get_elem_size(), self.RASTER_ELEM_SIZE)
 
     def test_get_pixel(self):
-        self.assertAlmostEqual(self.raster.get_pixel(0, 0), self.TEST_DATA[0, 0])
+        self.assertAlmostEqual(self.raster.get_pixel(0, 0),
+                               self.TEST_DATA[0, 0])
 
     def test_get_pixel_invalid_x(self):
         self.assertRaises(ValueError, self.raster.get_pixel, -1, 0)
@@ -776,19 +888,22 @@ class TestRaster(unittest.TestCase):
 
     def test_source_width_property(self):
         self.assertEqual(self.raster.source_width, self.RASTER_WIDTH)
-        self.assertTrue(isinstance(self.raster.source_width, (int, long)))
+        self.assertTrue(isinstance(self.raster.source_width, numbers.Integral))
 
     def test_source_height_property(self):
         self.assertEqual(self.raster.source_height, self.RASTER_HEIGHT)
-        self.assertTrue(isinstance(self.raster.source_height, (int, long)))
+        self.assertTrue(isinstance(self.raster.source_height,
+                                   numbers.Integral))
 
     def test_source_step_x_property(self):
         self.assertEqual(self.raster.source_step_x, 1)
-        self.assertTrue(isinstance(self.raster.source_step_x, (int, long)))
+        self.assertTrue(isinstance(self.raster.source_step_x,
+                                   numbers.Integral))
 
     def test_source_step_y_property(self):
         self.assertEqual(self.raster.source_step_y, 1)
-        self.assertTrue(isinstance(self.raster.source_step_y, (int, long)))
+        self.assertTrue(isinstance(self.raster.source_step_y,
+                                   numbers.Integral))
 
     def test_data_property(self):
         height = self.raster.get_height()
@@ -834,7 +949,7 @@ class TestRaster(unittest.TestCase):
 
 
 class TestRasterRead(TestRaster):
-    PRODUCT_FILE = TEST_PRODUCT
+    PRODUCT_FILE = os.path.join(TESTDIR, TEST_PRODUCT)
     BAND_NAME = TestBand.BAND_NAME
     RASTER_XOFFSET = TestBand.XOFFSET
     RASTER_YOFFSET = TestBand.YOFFSET
@@ -858,6 +973,7 @@ class TestRasterRead(TestRaster):
         self.assertEqual(data1[0, 0], data2[0, 0])
         self.assertTrue(np.all(data1 == data2))
 
+
 class TestRasterHighLevelAPI(unittest.TestCase):
     RASTER_WIDTH = 400
     RASTER_HEIGHT = 300
@@ -877,9 +993,15 @@ class TestRasterHighLevelAPI(unittest.TestCase):
         self.assertEqual(mobj.group('lines'), str(self.raster.get_height()))
         self.assertEqual(mobj.group('pixels'), str(self.raster.get_width()))
 
+    def test_repr_type(self):
+        self.assertTrue(isinstance(repr(self.raster), str))
+
+    def test_str_type(self):
+        self.assertTrue(isinstance(str(self.raster), str))
+
 
 class TestRecord(unittest.TestCase):
-    PRODUCT_FILE = TEST_PRODUCT
+    PRODUCT_FILE = os.path.join(TESTDIR, TEST_PRODUCT)
     DATASET_NAME = 'Quality_ADS'
     NUM_FIELD = 21
     FIELD_NAME = 'perc_water_abs_aero'
@@ -912,7 +1034,8 @@ class TestRecord(unittest.TestCase):
         self.record.print_element(0, 0, sys.stderr)
 
     def test_print_element_invalid_ostream(self):
-        self.assertRaises(TypeError, self.record.print_element, 0, 0, 'invalid')
+        self.assertRaises(TypeError, self.record.print_element, 0, 0,
+                          'invalid')
 
     def test_print_element_field_out_of_range(self):
         index = self.record.get_num_fields() + 10
@@ -925,6 +1048,20 @@ class TestRecord(unittest.TestCase):
         field = self.record.get_field(self.FIELD_NAME)
         self.assertTrue(isinstance(field, epr.Field))
 
+    if 'unicode' in dir(__builtins__):
+
+        def test_get_field_unicode(self):
+            field = self.record.get_field(unicode(self.FIELD_NAME))
+            self.assertTrue(isinstance(field, epr.Field))
+
+    else:
+
+        def test_get_field_bytes(self):
+            field_name = self.FIELD_NAME.encode('UTF-8')
+            field = self.record.get_field(self.FIELD_NAME)
+            self.assertTrue(isinstance(field_name, bytes))
+            self.assertTrue(isinstance(field, epr.Field))
+
     def test_get_field_invlid_name(self):
         self.assertRaises(ValueError, self.record.get_field, '')
 
@@ -937,7 +1074,7 @@ class TestRecord(unittest.TestCase):
 
 
 class TestRecordHighLevelAPI(unittest.TestCase):
-    PRODUCT_FILE = TEST_PRODUCT
+    PRODUCT_FILE = os.path.join(TESTDIR, TEST_PRODUCT)
     DATASET_NAME = TestRecord.DATASET_NAME
     FIELD_NAMES = [
         'dsr_time',
@@ -991,9 +1128,15 @@ class TestRecordHighLevelAPI(unittest.TestCase):
             index += 1
         self.assertEqual(index, self.record.get_num_fields())
 
+    def test_repr_type(self):
+        self.assertTrue(isinstance(repr(self.record), str))
+
+    def test_str_type(self):
+        self.assertTrue(isinstance(str(self.record), str))
+
 
 class TestMultipleRecordsHighLevelAPI(unittest.TestCase):
-    PRODUCT_FILE = TEST_PRODUCT
+    PRODUCT_FILE = os.path.join(TESTDIR, TEST_PRODUCT)
     DATASET_NAME = TestProduct.DATASET_NAME
 
     def setUp(self):
@@ -1009,7 +1152,7 @@ class TestMultipleRecordsHighLevelAPI(unittest.TestCase):
 
     def test_str_vs_print(self):
         for record in self.dataset:
-            with tempfile.TemporaryFile() as fd:
+            with tempfile.TemporaryFile('w+') as fd:
                 record.print_(fd)
                 fd.flush()
                 fd.seek(0)
@@ -1020,7 +1163,7 @@ class TestMultipleRecordsHighLevelAPI(unittest.TestCase):
 
 
 class TestMphRecordHighLevelAPI(TestRecordHighLevelAPI):
-    PRODUCT_FILE = TEST_PRODUCT
+    PRODUCT_FILE = os.path.join(TESTDIR, TEST_PRODUCT)
     DATASET_NAME = 'MPH'
     FIELD_NAMES = [
         'PRODUCT',
@@ -1041,8 +1184,9 @@ class TestMphRecordHighLevelAPI(TestRecordHighLevelAPI):
         product = epr.Product(self.PRODUCT_FILE)
         self.record = product.get_mph()
 
+
 class TestField(unittest.TestCase):
-    PRODUCT_FILE = TEST_PRODUCT
+    PRODUCT_FILE = os.path.join(TESTDIR, TEST_PRODUCT)
     DATASET_NAME = 'Quality_ADS'
 
     FIELD_NAME = 'perc_water_abs_aero'
@@ -1116,7 +1260,7 @@ class TestFieldWithMiltipleElems(TestField):
 
 
 class TestFieldHighLevelAPI(unittest.TestCase):
-    PRODUCT_FILE = TEST_PRODUCT
+    PRODUCT_FILE = os.path.join(TESTDIR, TEST_PRODUCT)
     DATASET_NAME = TestProduct.DATASET_NAME
 
     def setUp(self):
@@ -1135,9 +1279,17 @@ class TestFieldHighLevelAPI(unittest.TestCase):
             self.assertEqual(mobj.group('type'),
                              epr.data_type_id_to_str(field.get_type()))
 
+    def test_repr_type(self):
+        field = self.record.get_field_at(0)
+        self.assertTrue(isinstance(repr(field), str))
+
+    def test_str_type(self):
+        field = self.record.get_field_at(0)
+        self.assertTrue(isinstance(str(field), str))
+
     def test_str_vs_print(self):
         for field in self.record:
-            with tempfile.TemporaryFile() as fd:
+            with tempfile.TemporaryFile('w+') as fd:
                 field.print_(fd)
                 fd.flush()
                 fd.seek(0)
@@ -1174,7 +1326,7 @@ class TestFieldHighLevelAPI(unittest.TestCase):
 
 
 class TestFieldHighLevelAPI2(unittest.TestCase):
-    PRODUCT_FILE = TEST_PRODUCT
+    PRODUCT_FILE = os.path.join(TESTDIR, TEST_PRODUCT)
 
     def setUp(self):
         self.product = epr.Product(self.PRODUCT_FILE)
@@ -1205,48 +1357,50 @@ class TestFieldHighLevelAPI2(unittest.TestCase):
 
 
 class TestDSD(unittest.TestCase):
-    PRODUCT_FILE = TEST_PRODUCT
+    PRODUCT_FILE = os.path.join(TESTDIR, TEST_PRODUCT)
+    DSD_INDEX = 0
     DS_NAME = 'Quality ADS'
     DS_OFFSET = 12869
+    DS_TYPE = 'A'
     DS_SIZE = 160
     DSR_SIZE = 32
     NUM_DSR = 5
 
     def setUp(self):
         self.product = epr.Product(self.PRODUCT_FILE)
-        self.dsd = self.product.get_dsd_at(0)
+        self.dsd = self.product.get_dsd_at(self.DSD_INDEX)
 
     def test_index(self):
-        self.assertEqual(self.dsd.index, 0)
+        self.assertEqual(self.dsd.index, self.DSD_INDEX)
         self.assertTrue(isinstance(self.dsd.index, int))
 
     def test_ds_name(self):
         self.assertEqual(self.dsd.ds_name, self.DS_NAME)
-        self.assertTrue(isinstance(self.dsd.ds_name, basestring))
+        self.assertTrue(isinstance(self.dsd.ds_name, str))
 
     def test_ds_type(self):
-        self.assertEqual(self.dsd.ds_type, 'A')
-        self.assertTrue(isinstance(self.dsd.ds_type, basestring))
+        self.assertEqual(self.dsd.ds_type, self.DS_TYPE)
+        self.assertTrue(isinstance(self.dsd.ds_type, str))
 
     def test_filename(self):
         self.assertEqual(self.dsd.filename, '')
-        self.assertTrue(isinstance(self.dsd.filename, basestring))
+        self.assertTrue(isinstance(self.dsd.filename, str))
 
     def test_ds_offset(self):
         self.assertEqual(self.dsd.ds_offset, self.DS_OFFSET)
-        self.assertTrue(isinstance(self.dsd.ds_offset, (int, long)))
+        self.assertTrue(isinstance(self.dsd.ds_offset, numbers.Integral))
 
     def test_ds_size(self):
         self.assertEqual(self.dsd.ds_size, self.DS_SIZE)
-        self.assertTrue(isinstance(self.dsd.ds_size, (int, long)))
+        self.assertTrue(isinstance(self.dsd.ds_size, numbers.Integral))
 
     def test_num_dsr(self):
         self.assertEqual(self.dsd.num_dsr, self.NUM_DSR)
-        self.assertTrue(isinstance(self.dsd.num_dsr, (int, long)))
+        self.assertTrue(isinstance(self.dsd.num_dsr, numbers.Integral))
 
     def test_dsr_size(self):
         self.assertEqual(self.dsd.dsr_size, self.DSR_SIZE)
-        self.assertTrue(isinstance(self.dsd.dsr_size, (int, long)))
+        self.assertTrue(isinstance(self.dsd.dsr_size, numbers.Integral))
 
     def test_eq_dsd1_dsd1(self):
         self.assertEqual(self.dsd, self.dsd)
@@ -1272,7 +1426,7 @@ class TestDSD(unittest.TestCase):
 
 
 class TestDsdHighLevelAPI(unittest.TestCase):
-    PRODUCT_FILE = TEST_PRODUCT
+    PRODUCT_FILE = os.path.join(TESTDIR, TEST_PRODUCT)
 
     def setUp(self):
         product = epr.Product(self.PRODUCT_FILE)
@@ -1284,6 +1438,12 @@ class TestDsdHighLevelAPI(unittest.TestCase):
         self.assertNotEqual(mobj, None)
         self.assertEqual(mobj.group('name'), self.dsd.ds_name)
 
+    def test_repr_type(self):
+        self.assertTrue(isinstance(repr(self.dsd), str))
+
+    def test_str_type(self):
+        self.assertTrue(isinstance(str(self.dsd), str))
+
 
 class TestDataypeFunctions(unittest.TestCase):
     TYPE_NAMES = {
@@ -1366,20 +1526,22 @@ class TestSampleModelFunctions(unittest.TestCase):
 class TestDirectInstantiation(unittest.TestCase):
     MSG_PATTERN = '"%s" class cannot be instantiated from Python'
 
-    if sys.version_info[:2] > (3,2):
+    if sys.version_info[:2] >= (3, 2):
         # @COMPATIBILITY: python >= 3.2
         pass
-    elif sys.version_info[:2] in ((2.7), (3,1)):
+    elif sys.version_info[:2] in ((2, 7), (3, 1)):
         # @COMPATIBILITY: unittest2, python2.7, python3.1
-        assertRaiserRegex = assertRaiserRegexp
+        assertRaisesRegex = unittest.TestCase.assertRaisesRegexp
     else:
+
         # @COMPATIBILITY: python < 2.7
         def assertRaisesRegex(self, expected_exception, expected_regexp,
                               callable_obj=None, *args, **kwargs):
             try:
                 callable_obj(*args, **kwargs)
-            except expected_exception, exc_value:
-                if isinstance(expected_regexp, basestring):
+            except expected_exception as exc_value:
+                import types
+                if isinstance(expected_regexp, types.StringTypes):
                     expected_regexp = re.compile(expected_regexp)
                 if not expected_regexp.search(str(exc_value)):
                     raise self.failureException('"%s" does not match "%s"' %
@@ -1389,7 +1551,7 @@ class TestDirectInstantiation(unittest.TestCase):
                     excName = expected_exception.__name__
                 else:
                     excName = str(expected_exception)
-                raise self.failureException, "%s not raised" % excName
+                raise self.failureException("%s not raised" % excName)
 
     def test_direct_dsd_instantiation(self):
         pattern = self.MSG_PATTERN % epr.DSD.__name__
@@ -1421,7 +1583,7 @@ class TestDirectInstantiation(unittest.TestCase):
 
 class TestLibVersion(unittest.TestCase):
     def test_c_api_version(self):
-        self.assertTrue(isinstance(epr.EPR_C_API_VERSION, basestring))
+        self.assertTrue(isinstance(epr.EPR_C_API_VERSION, str))
 
 
 if __name__ == '__main__':
